@@ -52,9 +52,9 @@ methods:
 
 Exceptions:
  UnknownClassicalModeError
- BadnumValueError
+ BadNumValueError
  BadChunkingOptionError
- numOutOfRangeError
+ NumOutOfRangeError
  BadUserDefinedPatternError
  BadRcFileError
 
@@ -68,9 +68,9 @@ from os.path import dirname, isfile, expanduser
 from os.path import join as pathjoin
 
 class UnknownClassicalModeError(Exception): pass
-class BadnumValueError(Exception): pass
+class BadNumValueError(Exception): pass
 class BadChunkingOptionError(Exception): pass
-class numOutOfRangeError(Exception): pass
+class NumOutOfRangeError(Exception): pass
 class BadUserDefinedPatternError(Exception): pass
 class BadRcFileError(Exception): pass
 
@@ -419,15 +419,15 @@ pl_sb_uninflected_s = [
     "chassis", "innings", "news", "mews", "haggis",
 ]
 
-pl_sb_uninflected_herd = enclose('|'.join((
+pl_sb_uninflected_herd = (
 # DON'T INFLECT IN CLASSICAL MODE, OTHERWISE NORMAL INFLECTION
     "wildebeest", "swine", "eland", "bison", "buffalo",
     "elk", "rhinoceros", 'zucchini',
-    'caribou', 'dace', 'grouse', 'guinea[- ]fowl',
+    'caribou', 'dace', 'grouse', 'guinea fowl', 'guinea-fowl',
     'haddock', 'hake', 'halibut', 'herring', 'mackerel',
     'pickerel', 'pike', 'roe', 'seed', 'shad',
-    'snipe', 'teal', 'turbot', 'water[- ]fowl',
-)))
+    'snipe', 'teal', 'turbot', 'water fowl', 'water-fowl',
+)
 
 pl_sb_uninflected = enclose('|'.join ([
 # SOME FISH AND HERD ANIMALS
@@ -591,9 +591,9 @@ plverb_ambiguous_non_pres = enclose('|'.join ((
 
 # "..oes" -> "..oe" (the rest are "..oes" -> "o")
 
-plverb_oes_oe = ('canoes', 'floes', 'oboes', 'roes', 'throes', 'woes')
-plverb_oes_oe_endings_size4 = ('hoes', 'toes')
-plverb_oes_oe_endings_size5 = ('shoes')
+pl_v_oes_oe = ('canoes', 'floes', 'oboes', 'roes', 'throes', 'woes')
+pl_v_oes_oe_endings_size4 = ('hoes', 'toes')
+pl_v_oes_oe_endings_size5 = ('shoes')
 
 
 pl_count_zero = (
@@ -605,14 +605,14 @@ pl_count_one = (
 "1", "a", "an", "one", "each", "every", "this", "that",
 )
 
-pladj_special = {
+pl_adj_special = {
 "a"    : "some",   "an"   :  "some",
 "this" : "these",  "that" : "those",
 }
 
-pladj_special_keys = enclose('|'.join(pladj_special.keys()))
+pl_adj_special_keys = enclose('|'.join(pl_adj_special.keys()))
 
-pladj_poss = {
+pl_adj_poss = {
 "my"    : "our",
 "your"  : "your",
 "its"   : "their",
@@ -621,7 +621,7 @@ pladj_poss = {
 "their" : "their",
 }
 
-pladj_poss_keys = enclose('|'.join(pladj_poss.keys()))
+pl_adj_poss_keys = enclose('|'.join(pl_adj_poss.keys()))
 
 
 # 2. INDEFINITE ARTICLES
@@ -651,7 +651,7 @@ A_explicit_an = enclose('|'.join((
     )))
 
 
-# numERICAL INFLECTIONS
+# NUMERICAL INFLECTIONS
 
 nth = {
      0: 'th',
@@ -683,17 +683,7 @@ ordinal_suff = '|'.join(ordinal.keys())
 
 
 
-# numBERS
-
-default_args = {
-    'group'   : 0,
-    'comma'   : ',',
-    'andword'   : 'and',
-    'zero'    : 'zero',
-    'one'     : 'one',
-    'decimal' : 'point',
-}
-
+# NUMBERS
 
 unit = ['', 'one', 'two', 'three', 'four', 'five',
         'six', 'seven', 'eight', 'nine']
@@ -742,44 +732,62 @@ class engine:
 
         self.classical_dict = def_classical.copy()
         self.persistent_count = None
-        self.number_args = default_args.copy() # user can overwrite these in num
         self.mill_count = 0
         self.pl_sb_user_defined = []
-        self.plverb_user_defined  = []
-        self.pladj_user_defined  = []
+        self.pl_v_user_defined  = []
+        self.pl_adj_user_defined  = []
         self.A_a_user_defined   = []
 
     def defnoun(self, singular, plural):
+        '''
+        Set the noun plural of singular to plural.
+
+        '''
         self.checkpat(singular)
         self.checkpatplural(plural)
         self.pl_sb_user_defined.extend((singular, plural))
         return 1
 
-    # TODO does the eg in the docs work? will goes to both shall and will?
-    # will it work for the test of what is the plurl of what
-
     def defverb(self, s1, p1, s2, p2, s3, p3):
+        '''
+        Set the verb plurals for s1, s2 and s3 to p1, p2 and p3 respectively.
+
+        Where 1, 2 and 3 represent the 1st, 2nd and 3rd person forms of the verb.
+
+        '''
         self.checkpat(s1)
         self.checkpat(s2)
         self.checkpat(s3)
         self.checkpatplural(p1)
         self.checkpatplural(p2)
         self.checkpatplural(p3)
-        self.plverb_user_defined.extend((s1, p1, s2, p2, s3, p3))
+        self.pl_v_user_defined.extend((s1, p1, s2, p2, s3, p3))
         return 1
 
     def defadj(self, singular, plural):
+        '''
+        Set the adjective plural of singular to plural.
+
+        '''
         self.checkpat(singular)
         self.checkpatplural(plural)
-        self.pladj_user_defined.extend((singular, plural))
+        self.pl_adj_user_defined.extend((singular, plural))
         return 1
 
     def defa(self, pattern):
+        '''
+        Define the indefinate article as 'a' for words matching pattern.
+
+        '''
         self.checkpat(pattern)
         self.A_a_user_defined.extend((pattern, 'a'))
         return 1
 
     def defan(self, pattern):
+        '''
+        Define the indefinate article as 'an' for words matching pattern.
+
+        '''
         self.checkpat(pattern)
         self.A_a_user_defined.extend((pattern, 'an'))
         return 1
@@ -824,10 +832,10 @@ class engine:
 
     def classical(self, *args, **kwargs):
         """
-        Set the classical mode by changing classical_dict
+        Change the classical mode.
 
-        no args: all_classical
-        single args: all_classical if arg else no_classical
+        no args: all classical
+        single args: all classical if arg else no classical
         if 'all' in args or all=<true value> then all_classical
         if all=<false value> then no_classical
         else set each that appears in args as long as it is a valid key
@@ -856,11 +864,19 @@ class engine:
 
 
     def num(self, count=None, show=None):     # (;$count,$show)
+        '''
+        Set the number to be used in other method calls.
+
+        Returns count.
+
+        Set show to False to return '' instead.
+
+        '''
         if count is not None:
             try:
                 self.persistent_count = int(count)
             except ValueError:
-                raise BadnumValueError
+                raise BadNumValueError
             if (show is None) or show:
                 return str(count)
         else:
@@ -902,7 +918,7 @@ class engine:
         '''
         return self.pladj(matchobject.group(1), matchobject.group(3))
 
-    def Amo(self, matchobject):
+    def amo(self, matchobject):
         '''
         A but take a matchobject
         use groups 1 and 3 in matchobject
@@ -911,7 +927,7 @@ class engine:
             return self.a(matchobject.group(1))            
         return self.a(matchobject.group(1), matchobject.group(3))
 
-    def NOmo(self, matchobject):
+    def nomo(self, matchobject):
         '''
         NO but take a matchobject
         use groups 1 and 3 in matchobject
@@ -943,7 +959,16 @@ class engine:
 # 0. PERFORM GENERAL INFLECTIONS IN A STRING
 
     def inflect(self, text):
+        '''
+        Perform inflections in a string.
 
+        e.g. inflect('The plural of cat is pl(cat)') returns
+        'The plural of cat is cats'
+
+        can use pl, plnoun, plverb, pladj, a, an, no, ordinal,
+        numwords and prespart
+
+        '''
         save_persistent_count = self.persistent_count
         sections = splitre(r"(num\([^)]*\))", text)
         inflection = []
@@ -970,11 +995,11 @@ class engine:
                     total += count
                     (section, count) = subn(
                         r"(?x)\ban?    \( ([^),]*) (, ([^)]*) )? \)  ",
-                        self.Amo, section)
+                        self.amo, section)
                     total += count
                     (section, count) = subn(
                         r"(?x)\bno    \( ([^),]*) (, ([^)]*) )? \)  ",
-                        self.NOmo, section)
+                        self.nomo, section)
                     total += count
                     (section, count) = subn(
                         r"(?x)\bordinal        \( ([^)]*) \)            ",
@@ -1011,60 +1036,147 @@ class engine:
                              inflected[1:])
         return inflected
 
-    def partition_word(self, mystr):
-        mo = search(r'\A(\s*)(.+?)(\s*)\Z', mystr)
+    def partition_word(self, text):
+        mo = search(r'\A(\s*)(.+?)(\s*)\Z', text)
         try:
             return mo.group(1), mo.group(2), mo.group(3)
         except AttributeError: # empty string
             return '', '', ''
 
-    def pl(self, mystr, count=None):
-        pre, word, post = self.partition_word(mystr)
+    def pl(self, text, count=None):
+        '''
+        Return the plural of text.
+
+        If count supplied, then return text if count is one of:
+            1, a, an, one, each, every, this, that
+        otherwise return the plural.
+
+        Whitespace at the start and end is preserved.
+        
+        '''
+        pre, word, post = self.partition_word(text)
         if not word:
-            return mystr
+            return text
         plural = self.postprocess(word,
               self._pl_special_adjective(word, count)
               or self._pl_special_verb(word, count)
-              or self._plnounoun(word, count))
+              or self._plnoun(word, count))
         return "%s%s%s" % (pre, plural, post)
 
-    def plnoun(self, mystr, count=None):
-        pre, word, post = self.partition_word(mystr)
+    def plnoun(self, text, count=None):
+        '''
+        Return the plural of text, where text is a noun.
+
+        If count supplied, then return text if count is one of:
+            1, a, an, one, each, every, this, that
+        otherwise return the plural.
+
+        Whitespace at the start and end is preserved.
+        
+        '''
+        pre, word, post = self.partition_word(text)
         if not word:
-            return mystr
-        plural = self.postprocess(word, self._plnounoun(word, count))
+            return text
+        plural = self.postprocess(word, self._plnoun(word, count))
         return "%s%s%s" % (pre, plural, post)
 
-    def plverb(self, mystr, count=None):
-        pre, word, post = self.partition_word(mystr)
+    def plverb(self, text, count=None):
+        '''
+        Return the plural of text, where text is a verb.
+
+        If count supplied, then return text if count is one of:
+            1, a, an, one, each, every, this, that
+        otherwise return the plural.
+
+        Whitespace at the start and end is preserved.
+        
+        '''
+        pre, word, post = self.partition_word(text)
         if not word:
-            return mystr
+            return text
         plural = self.postprocess(word, self._pl_special_verb(word, count)
               or self._pl_general_verb(word, count))
         return "%s%s%s" % (pre, plural, post)
 
-    def pladj(self, mystr, count=None):
-        pre, word, post = self.partition_word(mystr)
+    def pladj(self, text, count=None):
+        '''
+        Return the plural of text, where text is an adjective.
+
+        If count supplied, then return text if count is one of:
+            1, a, an, one, each, every, this, that
+        otherwise return the plural.
+
+        Whitespace at the start and end is preserved.
+        
+        '''
+        pre, word, post = self.partition_word(text)
         if not word:
-            return mystr
+            return text
         plural = self.postprocess(word, self._pl_special_adjective(word, count)
               or word)
         return "%s%s%s" % (pre, plural, post)
 
     def plequal(self, word1, word2):
+        '''
+        compare word1 and word2 for equality regardless of plurality
+
+        return values:
+        eq - the strings are equal
+        p:s - word1 is the plural of word2
+        s:p - word2 is the plural of word1
+        p:p - word1 and word2 are two different plural forms of the one word
+        False - otherwise
+
+        '''
         return (
           self._plequal(word1, word2, self.plnoun) or
           self._plequal(word1, word2, self.plverb) or
           self._plequal(word1, word2, self.pladj))
 
     def plnounequal(self, word1, word2):
-          return self._plequal(word1, word2, self.plnoun)
+        '''
+        compare word1 and word2 for equality regardless of plurality
+        word1 and word2 are to be treated as nouns
+
+        return values:
+        eq - the strings are equal
+        p:s - word1 is the plural of word2
+        s:p - word2 is the plural of word1
+        p:p - word1 and word2 are two different plural forms of the one word
+        False - otherwise
+
+        '''
+        return self._plequal(word1, word2, self.plnoun)
 
     def plverbequal(self, word1, word2):
-          return self._plequal(word1, word2, self.plverb)
+        '''
+        compare word1 and word2 for equality regardless of plurality
+        word1 and word2 are to be treated as verbs
+
+        return values:
+        eq - the strings are equal
+        p:s - word1 is the plural of word2
+        s:p - word2 is the plural of word1
+        p:p - word1 and word2 are two different plural forms of the one word
+        False - otherwise
+
+        '''
+        return self._plequal(word1, word2, self.plverb)
 
     def pladjequal(self, word1, word2):
-          return self._plequal(word1, word2, self.pladj)
+        '''
+        compare word1 and word2 for equality regardless of plurality
+        word1 and word2 are to be treated as adjectives
+
+        return values:
+        eq - the strings are equal
+        p:s - word1 is the plural of word2
+        s:p - word2 is the plural of word1
+        p:p - word1 and word2 are two different plural forms of the one word
+        False - otherwise
+
+        '''
+        return self._plequal(word1, word2, self.pladj)
 
     def _plequal(self, word1, word2, pl):
         classval = self.classical_dict.copy()
@@ -1088,21 +1200,15 @@ class engine:
             if self._pl_check_plurals_N(word2, word1):
                 return "p:p"
         if pl == self.pl or pl == self.pladj:
-            if self._pl_check_plurals_ADJ(word1, word2):
+            if self._pl_check_plurals_adj(word1, word2):
                 return "p:p"
         return False
 
     def _pl_reg_plurals(self, pair, stems, end1, end2):
-#TODO delete commented try block
-      #try:
         if search(r"(%s)(%s\|\1%s|%s\|\1%s)" %
                                  (stems, end1, end2, end2, end1),
                                          pair):
             return True
-      #except:
-        #print r"(%s)(%s\|\1%s|%s\|\1%s)" % (stems, end1, end2, end2, end1), pair
-
-        #raise
         return False
 
     def _pl_check_plurals_N(self, word1, word2):
@@ -1134,29 +1240,32 @@ class engine:
                 return True
         return False
 
-    def _pl_check_plurals_ADJ(self, word1, word2):
+    def _pl_check_plurals_adj(self, word1, word2):
 #VERSION: tuple in endswith requires python 2.5
         word1a = word1[:word1.rfind("'")] if word1.endswith(("'s","'")) else ''
         word2a = word2[:word2.rfind("'")] if word2.endswith(("'s","'")) else ''
         #TODO: BUG? report upstream. I don't think you should chop off the s'
-        word1b = word1[:-2] if word1.endswith("s'") else ''
-        word2b = word2[:-2] if word2.endswith("s'") else ''
+        #word1b = word1[:-2] if word1.endswith("s'") else ''
+        #word2b = word2[:-2] if word2.endswith("s'") else ''
+
+        #TODO: dresses', dresses's -> dresses, dresses when chop off letters
+        # then they return False because they are the same. Need to fix this.
 
         if word1a:
             if word2a and ( self._pl_check_plurals_N(word1a, word2a)
                             or self._pl_check_plurals_N(word2a, word1a) ):
                 return True
-            if word2b and ( self._pl_check_plurals_N(word1a, word2b)
-                            or self._pl_check_plurals_N(word2b, word1a) ):
-                return True
+        #    if word2b and ( self._pl_check_plurals_N(word1a, word2b)
+        #                    or self._pl_check_plurals_N(word2b, word1a) ):
+        #        return True
 
-        if word1b:
-            if word2a and ( self._pl_check_plurals_N(word1b, word2a)
-                            or self._pl_check_plurals_N(word2a, word1b) ):
-                return True
-            if word2b and ( self._pl_check_plurals_N(word1b, word2b)
-                            or self._pl_check_plurals_N(word2b, word1b) ):
-                return True
+        #if word1b:
+        #    if word2a and ( self._pl_check_plurals_N(word1b, word2a)
+        #                    or self._pl_check_plurals_N(word2a, word1b) ):
+        #        return True
+        #    if word2b and ( self._pl_check_plurals_N(word1b, word2b)
+        #                    or self._pl_check_plurals_N(word2b, word1b) ):
+        #        return True
         
         return False
 
@@ -1172,7 +1281,7 @@ class engine:
             count = ''
         return count
 
-    def _plnounoun(self, word, count=None):
+    def _plnoun(self, word, count=None):
         count = self.get_count(count)
 
 # DEFAULT TO PLURAL
@@ -1187,37 +1296,38 @@ class engine:
             return value
 
 
-# HANDLE EMPTY Wordinal, SINGULAR COUNT AND UNINFLECTED PLURALS
+# HANDLE EMPTY WORD, SINGULAR COUNT AND UNINFLECTED PLURALS
 
         if word == '':
             return word
 
+        lowerword = word.lower()
+
         if search(r"^%s$" % pl_sb_uninflected, word, IGNORECASE):
             return word
 
         if (self.classical_dict['herd'] and
-               search(r"^%s$" % pl_sb_uninflected_herd, word, IGNORECASE)):
+               lowerword in pl_sb_uninflected_herd):
             return word
 
 # HANDLE COMPOUNDS ("Governor General", "mother-in-law", "aide-de-camp", ETC.)
 
         mo = search(r"^(?:%s)$" % pl_sb_postfix_adj_stems, word, IGNORECASE)
         if mo and mo.group(2) != '':
-            return "%s%s" % (self._plnounoun(mo.group(1), 2), mo.group(2))
+            return "%s%s" % (self._plnoun(mo.group(1), 2), mo.group(2))
 
         mo = search(r"^(?:%s)$" % pl_sb_prep_dual_compound, word, IGNORECASE)
         if mo and mo.group(2) != '' and mo.group(3) != '':
-            return "%s%s%s" % (self._plnounoun(mo.group(1), 2),
+            return "%s%s%s" % (self._plnoun(mo.group(1), 2),
                                mo.group(2),
-                               self._plnounoun(mo.group(3)))
+                               self._plnoun(mo.group(3)))
 
         mo = search(r"^(?:%s)$" % pl_sb_prep_compound, word, IGNORECASE)
         if mo and mo.group(2) != '':
-                return "%s%s" % (self._plnounoun(mo.group(1), 2), mo.group(2))
+                return "%s%s" % (self._plnoun(mo.group(1), 2), mo.group(2))
 
 # HANDLE PRONOUNS
 
-# TODO: BUG does not keep case: "about ME" -> "about us". bug not fixed here
         mo = search(r"^((?:%s)\s+)(%s)$" % (pl_prep, pl_pron_acc_keys), word,
                                                          IGNORECASE)
         if mo:
@@ -1325,17 +1435,21 @@ class engine:
         if mo:
             return "%ses" % mo.group(1)
 
-# TODO: not sure if this makes a difference. Wouldn't special words
+# Wouldn't special words
 # ending with 's' always have been caught, regardless of them starting
 # with a capital letter (i.e. being names)
 # It makes sense below to do this for words ending in 'y' so that
 # Sally -> Sallys. But not sure it makes sense here. Where is the case
 # of a word ending in s that is caught here and would otherwise have been
 # caught below?
-        if (self.classical_dict['names']):
-            mo = search(r"([A-Z].*s)$", word)
-            if mo:
-                return "%ses" % mo.group(1)
+#
+# removing it as I can't find a case that executes it
+#
+#
+#        if (self.classical_dict['names']):
+#            mo = search(r"([A-Z].*s)$", word)
+#            if mo:
+#                return "%ses" % mo.group(1)
 
         mo = search(r"^(.*[^z])(z)$", word, IGNORECASE)
         if mo:
@@ -1395,17 +1509,18 @@ class engine:
 
 
     def _pl_special_verb(self, word, count=None):
+        
+        if (self.classical_dict['zero'] and 
+            str(count).lower() in pl_count_zero):
+                return False
         count = self.get_count(count)
-# TODO: should zero make count 2 not return here: check Perl code
-        if str(count).lower() in pl_count_zero:
-            return False
 
         if count==1:
             return word
 
 # HANDLE USER-DEFINED VERBS
 
-        value = self.ud_match(word, self.plverb_user_defined)
+        value = self.ud_match(word, self.pl_v_user_defined)
         if value is not None:
             return value
 
@@ -1456,9 +1571,9 @@ class engine:
         if mo:
             return "%sy" % mo.group(1)
 
-        if (lowerword in plverb_oes_oe or
-            lowerword[-4:] in plverb_oes_oe_endings_size4 or
-            lowerword[-5:] in plverb_oes_oe_endings_size5):
+        if (lowerword in pl_v_oes_oe or
+            lowerword[-4:] in pl_v_oes_oe_endings_size4 or
+            lowerword[-5:] in pl_v_oes_oe_endings_size5):
                 return word[:-1]
 
         if lowerword.endswith('oes') and len(lowerword)>3:
@@ -1507,23 +1622,23 @@ class engine:
 
 # HANDLE USER-DEFINED ADJECTIVES
 
-        value = self.ud_match(word, self.pladj_user_defined)
+        value = self.ud_match(word, self.pl_adj_user_defined)
         if value is not None:
             return value
 
 # HANDLE KNOWN CASES
 
-        mo = search(r"^(%s)$" % pladj_special_keys,
+        mo = search(r"^(%s)$" % pl_adj_special_keys,
                     word, IGNORECASE)
         if mo:
-            return "%s" % (pladj_special[mo.group(1).lower()])
+            return "%s" % (pl_adj_special[mo.group(1).lower()])
 
 # HANDLE POSSESSIVES
 
-        mo = search(r"^(%s)$" % pladj_poss_keys,
+        mo = search(r"^(%s)$" % pl_adj_poss_keys,
                     word, IGNORECASE)
         if mo:
-            return "%s" % (pladj_poss[mo.group(1).lower()])
+            return "%s" % (pl_adj_poss[mo.group(1).lower()])
 
         mo = search(r"^(.*)'s?$",
                     word)
@@ -1539,13 +1654,24 @@ class engine:
 
 # ADJECTIVES
 
-    def a(self, mystr, count=1):
+    def a(self, text, count=1):
+        '''
+        Return the appropriate indefinite article followed by text.
+
+        The indefinite article is either 'a' or 'an'.
+
+        If count is not one, then return count followed by text
+        instead of 'a' or 'an'.
+
+        Whitespace at the start and end is preserved.
+
+        '''
         mo = search(r"\A(\s*)(?:an?\s+)?(.+?)(\s*)\Z",
-                    mystr, IGNORECASE)
+                    text, IGNORECASE)
         if mo:
             word = mo.group(2)
             if not word:
-                return mystr
+                return text
             pre = mo.group(1)
             post = mo.group(3)
             result = self._indef_article(word, count)
@@ -1633,13 +1759,28 @@ class engine:
 
 # 2. TRANSLATE ZERO-QUANTIFIED $word TO "no pl($word)"
 
-    def no(self, mystr, count=None):
+    def no(self, text, count=None):
+        '''
+        If count is 0, no, zero or nil, return 'no' followed by the plural
+        of text.
+
+        If count is one of:
+            1, a, an, one, each, every, this, that
+        return count followed by text.
+
+        Otherwise return count follow by the plural of text.
+
+        In the return value count is always followed by a space.
+
+        Whitespace at the start and end is preserved.
+        
+        '''
         if count is None and self.persistent_count is not None:
             count = self.persistent_count
 
         if count is None:
             count = 0
-        mo = search(r"\A(\s*)(.+?)(\s*)\Z", mystr)
+        mo = search(r"\A(\s*)(.+?)(\s*)\Z", text)
         pre = mo.group(1)
         word = mo.group(2)
         post = mo.group(3)
@@ -1653,6 +1794,12 @@ class engine:
 # PARTICIPLES
 
     def prespart(self, word):
+        '''
+        Return the present participle for word.
+
+        word is the 3rd person singular verb.
+        
+        '''
         plv = self.plverb(word, 2)
 
         for pat, repl in (
@@ -1672,9 +1819,18 @@ class engine:
             
 
 
-# numERICAL INFLECTIONS
+# NUMERICAL INFLECTIONS
 
     def ordinal(self, num):
+        '''
+        Return the ordinal of num.
+
+        num can be an integer or text
+
+        e.g. ordinal(1) returns '1st'
+        ordinal('one') returns 'first'
+
+        '''
         if match(r"\d", num):
             n = int(num)
             try:
@@ -1694,9 +1850,8 @@ class engine:
     def millfn(self, ind=0):
         if ind > len(mill)-1:
             print3("number out of range")
-            raise numOutOfRangeError
-    
-        return mill[ind] if ind < len(mill) else ' ???illion' #TODO: never returns ???illion as this is caught above
+            raise NumOutOfRangeError
+        return mill[ind]
     
     
     def unitfn(self, units, mindex=0):
@@ -1830,16 +1985,32 @@ class engine:
         '''
         return ' '
 
-    def numwords(self, num, wantlist=False, **kwds):
-        self.number_args.update(kwds) 
-        group = int(self.number_args['group'])
+    def numwords(self, num, wantlist=False,
+                 group=0, comma=',', andword='and',
+                 zero='zero', one='one', decimal='point',
+                 threshold=None):
+        '''
+        Return a number in words.
+
+        group = 1, 2 or 3 to group numbers before turning into words
+        comma: define comma
+        andword: word for 'and'. Can be set to ''.
+            e.g. "one hundred and one" vs "one hundred one"
+        zero: word for '0'
+        one: word for '1'
+        decimal: word for decimal point
+        threshold: numbers above threshold not turned into words
+        
+        parameters not remembered from last call. Departure from Perl version.
+        '''
+        self.number_args = dict(andword=andword, zero=zero, one=one)
         num = '%s' % num
     
         # Handle "stylistic" conversions (up to a given threshold)...
-        if ('threshold' in self.number_args and
-             float(num) > self.number_args['threshold']):
+        if (threshold is not None and
+             float(num) > threshold):
             spnum = num.split('.',1)
-            while (self.number_args['comma']):
+            while (comma):
                     (spnum[0], n) = subn(r"(\d)(\d{3}(?:,|\Z))",r"\1,\2", spnum[0])
                     if n==0:
                         break
@@ -1857,13 +2028,11 @@ class engine:
             sign = "minus"
         else:
             sign = ""
-        comma = self.number_args['comma']
-        andword = self.number_args['andword'] #can't use 'and' keyword in **kwds
     
         myord =  (num[-2:] in ('st', 'nd', 'rd', 'th'))
         if myord:
             num = num[:-2]
-        if self.number_args['decimal']:
+        if decimal:
             if group != 0:
                 chunks = num.split('.')
             else:
@@ -1918,13 +2087,13 @@ class engine:
                 numchunks[-1] += 'th'
 
         for chunk in chunks[1:]:
-            numchunks.append(self.number_args['decimal'])
+            numchunks.append(decimal)
             numchunks.extend(chunk.split("%s " % comma))
 
         #wantlist: Perl list context. can explictly specify in Python
         if wantlist:
             if sign:
-                return [sign].append(numchunks)
+                numchunks = [sign] + numchunks
             return numchunks
         elif group:
             signout = "%s " % sign if sign else ''
@@ -1932,9 +2101,9 @@ class engine:
         else:
             signout = "%s " % sign if sign else ''
             num = "%s%s" % (signout, numchunks.pop(0))
-            first = not num.endswith(self.number_args['decimal'])
+            first = not num.endswith(decimal)
             for nc in numchunks:
-                if nc == self.number_args['decimal']:
+                if nc == decimal:
                     num += " %s" % nc
                     first = 0
                 elif first:
@@ -1949,6 +2118,10 @@ class engine:
                  final_sep=None,
                  conj='and', conj_spaced=True):
         '''
+        Join a words into a list.
+
+        e.g. wordlist(['ant', 'bee', 'fly']) returns 'ant, bee, and fly'
+        
         options:
         conj: replacement for 'and'
         sep: separator. default ',', unless ',' is in the list then ';'
