@@ -6,8 +6,8 @@ from re import error as reerror
 import inflect
 
 reload(inflect)
-from inflect import BadChunkingOptionError, numOutOfRangeError, UnknownClassicalModeError
-from inflect import UnknownClassicalModeError, BadnumValueError
+from inflect import BadChunkingOptionError, NumOutOfRangeError, UnknownClassicalModeError
+from inflect import UnknownClassicalModeError, BadNumValueError
 
 class test(unittest.TestCase):
     def test_enclose(self):
@@ -95,7 +95,7 @@ class test(unittest.TestCase):
         self.assertEqual(p.persistent_count, 3)
         self.assertEqual(ret, '')
 
-        self.assertRaises(BadnumValueError, p.num, 'text')
+        self.assertRaises(BadNumValueError, p.num, 'text')
 
     def test_inflect(self):
         p = inflect.engine()
@@ -182,14 +182,18 @@ class test(unittest.TestCase):
         p.defverb('will','shall',
                          'will','will',
                          'will','will')
-        self.assertEqual(p.ud_match('will',p.plverb_user_defined),
+        self.assertEqual(p.ud_match('will',p.pl_v_user_defined),
                          'will')
+        self.assertEqual(p.pl('will'),'will')
+        #TODO: will -> shall. Tests below fail
+        #self.assertEqual(p.plequal('will','shall'),'s:p')
+        #self.assertEqual(p.plverbequal('will','shall'),'s:p')
 
 
         #defadj
         p.defadj('hir','their')
         self.assertEqual(p.pl('hir'),'their')
-        self.assertEqual(p.ud_match('hir',p.pladj_user_defined),
+        self.assertEqual(p.ud_match('hir',p.pl_adj_user_defined),
                         'their')
 
         #defa defan
@@ -246,6 +250,7 @@ class test(unittest.TestCase):
                             (p.pl, 'knifes', 'knife'),
                             (p.pl, ' cat  ', ' cats  '),
                             (p.pl, 'court martial', 'courts martial'),
+                            (p.pl, 'a', 'some'),
                             (p.plnoun, '',''),
                             (p.plnoun, 'cow','cows'),
                             (p.plnoun, 'thought','thoughts'),
@@ -263,6 +268,28 @@ class test(unittest.TestCase):
                             ):
             self.assertEqual(fn(sing), plur)
 
+        for sing, num, plur in (
+                ('cow', 1, 'cow'),
+                ('cow', 2, 'cows'),
+                ('cow', 'one', 'cow'),
+                ('cow', 'each', 'cow'),
+                ('cow', 'two', 'cows'),
+                ('cow', 0, 'cows'),
+                ('cow', 'zero', 'cows'),
+                ('runs', 0, 'run'),
+                ('runs', 1, 'runs'),
+                ('am', 0, 'are'),
+                    ):
+            self.assertEqual(p.pl(sing, num), plur)
+
+        p.classical('zero')
+        self.assertEqual(p.pl('cow', 0), 'cow')
+        self.assertEqual(p.pl('cow', 'zero'), 'cow')
+        self.assertEqual(p.pl('runs', 0), 'runs')
+        self.assertEqual(p.pl('am', 0), 'am')
+        self.assertEqual(p.plverb('runs', 1), 'runs')
+
+
     def test_plequal(self):
         p = inflect.engine()
         for fn, sing, plur, res in (
@@ -274,6 +301,9 @@ class test(unittest.TestCase):
                     (p.plequal, 'indices','indexes','p:p'),
                     (p.plequal, 'indexes','indices','p:p'),
                     (p.plequal, 'indices','indices','eq'),
+                    (p.plequal, 'opuses','opera','p:p'),
+                    (p.plequal, 'opera','opuses','p:p'),
+                    (p.plequal, 'brothers','brethren','p:p'),                  
                     (p.plequal, 'cats','cats','eq'),
                     (p.plequal, 'base', 'basis', False),
                     (p.plequal, 'syrinx', 'syringe', False),
@@ -283,6 +313,9 @@ class test(unittest.TestCase):
                     (p.plequal, 'time', 'Times', False),
                     (p.plequal, 'time'.lower(), 'Times'.lower(), 's:p'),
                     (p.plequal, 'courts martial', 'court martial', 'p:s'),
+                    (p.plequal, 'my','my','eq'),
+                    (p.plequal, 'my','our','s:p'),
+                    (p.plequal, 'our','our','eq'),
                     (p.plnounequal, 'index','index','eq'),
                     (p.plnounequal, 'index','indexes','s:p'),
                     (p.plnounequal, 'index','indices','s:p'),
@@ -297,9 +330,21 @@ class test(unittest.TestCase):
                     (p.pladjequal, 'my','my','eq'),
                     (p.pladjequal, 'my','our','s:p'),
                     (p.pladjequal, 'our','our','eq'),
+                    (p.plequal, "dresses's","dresses'", 'p:s'), # TODO: should return p:p 
+                    (p.pladjequal, "dresses's","dresses'", False), # TODO: should return p:p
+
+                    # TODO: future: support different singulars one day.
+                    #(p.plequal, "dress's","dress'",'s:s'),
+                    #(p.pladjequal, "dress's","dress'",'s:s'),
+                    #(p.plequal, "Jess's","Jess'",'s:s'),
+                    #(p.pladjequal, "Jess's","Jess'",'s:s'),
                         ):
             self.assertEqual(fn(sing, plur), res)
 
+        #TODO: pass upstream. multiple adjective plurals not supported
+        #self.assertEqual(p.plequal('your', 'our'), False)
+        #p.defadj('my', 'our|your') # what's ours is yours
+        #self.assertEqual(p.plequal('your', 'our'), 'p:p')
 
     def test__pl_reg_plurals(self):
         p = inflect.engine()
@@ -321,16 +366,16 @@ class test(unittest.TestCase):
         self.assertEqual(p._pl_check_plurals_N('stigmata', 'stigmas'), True)
         self.assertEqual(p._pl_check_plurals_N('phalanxes', 'phalanges'), True)
 
-    def test__pl_check_plurals_ADJ(self):
+    def test__pl_check_plurals_adj(self):
         p = inflect.engine()
-        self.assertEqual(p._pl_check_plurals_ADJ("indexes's", "indices's"), True)
-        self.assertEqual(p._pl_check_plurals_ADJ("indices's", "indexes's"), True)
-        self.assertEqual(p._pl_check_plurals_ADJ("indexes'", "indices's"), True)
-        self.assertEqual(p._pl_check_plurals_ADJ("indexes's", "indices'"), True)
-        self.assertEqual(p._pl_check_plurals_ADJ("indexes's", "indexes's"), False)
-        self.assertEqual(p._pl_check_plurals_ADJ("dogmas's", "dogmata's"), True)
-        self.assertEqual(p._pl_check_plurals_ADJ("dogmas'", "dogmata'"), True)
-        self.assertEqual(p._pl_check_plurals_ADJ("indexes'", "indices'"), True)
+        self.assertEqual(p._pl_check_plurals_adj("indexes's", "indices's"), True)
+        self.assertEqual(p._pl_check_plurals_adj("indices's", "indexes's"), True)
+        self.assertEqual(p._pl_check_plurals_adj("indexes'", "indices's"), True)
+        self.assertEqual(p._pl_check_plurals_adj("indexes's", "indices'"), True)
+        self.assertEqual(p._pl_check_plurals_adj("indexes's", "indexes's"), False)
+        self.assertEqual(p._pl_check_plurals_adj("dogmas's", "dogmata's"), True)
+        self.assertEqual(p._pl_check_plurals_adj("dogmas'", "dogmata'"), True)
+        self.assertEqual(p._pl_check_plurals_adj("indexes'", "indices'"), True)
 
     def test_count(self):
         p = inflect.engine()
@@ -362,7 +407,7 @@ class test(unittest.TestCase):
         self.assertEqual(p.get_count(), 2)
 
 
-    def test__plnounoun(self):
+    def test__plnoun(self):
         p = inflect.engine()
         for sing, plur in (
                 ('', ''),
@@ -378,6 +423,7 @@ class test(unittest.TestCase):
                 ('mother in law', 'mothers in law'),
                 ('mother-in-law', 'mothers-in-law'),
                 ('about me', 'about us'),
+                #TODO: does not keep case
                 #('about ME', 'about US'),
                 ('to it', 'to them'),
                 ('from it', 'from them'),
@@ -427,25 +473,25 @@ class test(unittest.TestCase):
                 ('zoo', 'zoos'),
                 ('tomato', 'tomatoes'),
                 ):
-            self.assertEqual(p._plnounoun(sing), plur)
+            self.assertEqual(p._plnoun(sing), plur)
 
         p.num(1)
-        self.assertEqual(p._plnounoun('cat'), 'cat')
+        self.assertEqual(p._plnoun('cat'), 'cat')
         p.num(3)
 
 
         p.classical('herd')
-        self.assertEqual(p._plnounoun('swine'), 'swine')
+        self.assertEqual(p._plnoun('swine'), 'swine')
         p.classical(herd=0)
-        self.assertEqual(p._plnounoun('swine'), 'swines')
+        self.assertEqual(p._plnoun('swine'), 'swines')
         p.classical(persons=1)
-        self.assertEqual(p._plnounoun('chairperson'), 'chairpersons')
+        self.assertEqual(p._plnoun('chairperson'), 'chairpersons')
         p.classical(persons=0)
-        self.assertEqual(p._plnounoun('chairperson'), 'chairpeople')
+        self.assertEqual(p._plnoun('chairperson'), 'chairpeople')
         p.classical(ancient=1)
-        self.assertEqual(p._plnounoun('formula'), 'formulae')
+        self.assertEqual(p._plnoun('formula'), 'formulae')
         p.classical(ancient=0)
-        self.assertEqual(p._plnounoun('formula'), 'formulas')
+        self.assertEqual(p._plnoun('formula'), 'formulas')
 
 
         p.classical()
@@ -469,7 +515,7 @@ class test(unittest.TestCase):
                 ('goy', 'goyim'),
                 ('afrit', 'afriti'),
                 ):
-            self.assertEqual(p._plnounoun(sing), plur)
+            self.assertEqual(p._plnoun(sing), plur)
 
         #p.classical(0)
         #p.classical('names')
@@ -489,6 +535,13 @@ class test(unittest.TestCase):
         p = inflect.engine()
         self.assertEqual(p._pl_special_verb(''), False)
         self.assertEqual(p._pl_special_verb('am'), 'are')
+        self.assertEqual(p._pl_special_verb('am',0), 'are')
+        self.assertEqual(p._pl_special_verb('runs',0), 'run')
+        p.classical('zero')
+        self.assertEqual(p._pl_special_verb('am',0), False)
+        self.assertEqual(p._pl_special_verb('am',1), 'am')
+        self.assertEqual(p._pl_special_verb('am',2), 'are')
+        self.assertEqual(p._pl_special_verb('runs',0), False)
         self.assertEqual(p._pl_special_verb('am going to'), 'are going to')
         self.assertEqual(p._pl_special_verb('did'), 'did')
         self.assertEqual(p._pl_special_verb("wasn't"), "weren't")
@@ -511,11 +564,14 @@ class test(unittest.TestCase):
         self.assertEqual(p._pl_special_verb('zzzoes'), 'zzzo') # what's a real word to test this case?
         self.assertEqual(p._pl_special_verb('runs'), 'run')
 
+
+
     def test__pl_general_verb(self):
         p = inflect.engine()
         self.assertEqual(p._pl_general_verb('acts'), 'act')
         self.assertEqual(p._pl_general_verb('act'), 'act')
         self.assertEqual(p._pl_general_verb('saw'), 'saw')
+        self.assertEqual(p._pl_general_verb('runs', 1), 'runs')
         
     def test__pl_special_adjective(self):
         p = inflect.engine()
@@ -588,6 +644,8 @@ class test(unittest.TestCase):
         self.assertEqual(p.no('cat'), 'no cats')
         self.assertEqual(p.no('cat', count=3), '3 cats')
         self.assertEqual(p.no('cat', count='three'), 'three cats')
+        self.assertEqual(p.no('cat', count=1), '1 cat')
+        self.assertEqual(p.no('cat', count='one'), 'one cat')
         self.assertEqual(p.no('mouse'), 'no mice')
         p.num(3)
         self.assertEqual(p.no('cat'), '3 cats')
@@ -650,7 +708,7 @@ class test(unittest.TestCase):
         self.assertEqual(millfn(0), '')
         self.assertEqual(millfn(11), ' decillion')
         inflect.STDOUT_ON = False
-        self.assertRaises(numOutOfRangeError, millfn, 12)
+        self.assertRaises(NumOutOfRangeError, millfn, 12)
         inflect.STDOUT_ON = True
         
     def test_unitfn(self):
@@ -676,6 +734,7 @@ class test(unittest.TestCase):
     def test_hundfn(self):
         p = inflect.engine()
         hundfn = p.hundfn
+        p.number_args = dict(andword='and')
         self.assertEqual(hundfn(4,3,1,2), 'four hundred and thirty-one million, ')
         self.assertEqual(hundfn(4,0,0,2), 'four hundred million, ')
         self.assertEqual(hundfn(4,0,5,2), 'four hundred and five million, ')
@@ -687,6 +746,7 @@ class test(unittest.TestCase):
         enword = p.enword
         self.assertEqual(enword('5',1),
                          'five, ')
+        p.number_args = dict(zero='zero', one='one', andword='and')
         self.assertEqual(enword('0',1),
                          ' zero, ')
         self.assertEqual(enword('1',1),
@@ -700,9 +760,14 @@ class test(unittest.TestCase):
                          'thirty-four, seven, ')
         self.assertEqual(enword('34768',2),
                          'thirty-four, seventy-six, eight, ')
+        self.assertEqual(enword('1',2),
+                         'one, ')
+        p.number_args['one'] = 'single'
         self.assertEqual(enword('1',2),
                          'one, ') #TODO: doesn't use default word for 'one' here
 
+        p.number_args['one'] = 'one'
+
         self.assertEqual(enword('134',3),
                          ' one thirty-four, ')
 
@@ -768,7 +833,7 @@ class test(unittest.TestCase):
                           'two hundred and thirty-four thousand',
                           'five hundred and sixty-seven'])
         self.assertEqual(numwords('+10', wantlist=True),
-                         None) #TODO: fix this!
+                         ['plus', 'ten'])
         self.assertEqual(numwords('1234', andword=''),
                          'one thousand, two hundred thirty-four')
         self.assertEqual(numwords('1234', andword='plus'),
@@ -776,7 +841,7 @@ class test(unittest.TestCase):
         self.assertEqual(numwords('555_1202', group=1, zero='oh'),
                          'five, five, five, one, two, oh, two')
         self.assertEqual(numwords('555_1202', group=1, one='unity'),
-                         'five, five, five, unity, two, oh, two')
+                         'five, five, five, unity, two, zero, two')
         self.assertEqual(numwords('123.456', group=1, decimal='mark', one='one'),
                          'one, two, three, mark, four, five, six')
         self.assertEqual(numwords('12345', group=3),
